from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="ContentItemRegion")


@_attrs_define
class ContentItemRegion:
    """TODO

    Attributes:
        page_uid (str):
        coords (List[float]):
        is_empty (bool): TODO
        iiif_fragment (Union[Unset, str]): IIIF fragment URL
        g (Union[Unset, List[str]]): TODO
    """

    page_uid: str
    coords: List[float]
    is_empty: bool
    iiif_fragment: Union[Unset, str] = UNSET
    g: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        page_uid = self.page_uid

        coords = self.coords

        is_empty = self.is_empty

        iiif_fragment = self.iiif_fragment

        g: Union[Unset, List[str]] = UNSET
        if not isinstance(self.g, Unset):
            g = self.g

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "pageUid": page_uid,
                "coords": coords,
                "isEmpty": is_empty,
            }
        )
        if iiif_fragment is not UNSET:
            field_dict["iiifFragment"] = iiif_fragment
        if g is not UNSET:
            field_dict["g"] = g

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        page_uid = d.pop("pageUid")

        coords = cast(List[float], d.pop("coords"))

        is_empty = d.pop("isEmpty")

        iiif_fragment = d.pop("iiifFragment", UNSET)

        g = cast(List[str], d.pop("g", UNSET))

        content_item_region = cls(
            page_uid=page_uid,
            coords=coords,
            is_empty=is_empty,
            iiif_fragment=iiif_fragment,
            g=g,
        )

        return content_item_region



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="ContentItemMentionsItem")


@_attrs_define
class ContentItemMentionsItem:
    """
    Attributes:
        person (Union[Unset, List[List[int]]]):
        location (Union[Unset, List[List[int]]]):
    """

    person: Union[Unset, List[List[int]]] = UNSET
    location: Union[Unset, List[List[int]]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        person: Union[Unset, List[List[int]]] = UNSET
        if not isinstance(self.person, Unset):
            person = []
            for person_item_data in self.person:
                person_item = person_item_data

                person.append(person_item)

        location: Union[Unset, List[List[int]]] = UNSET
        if not isinstance(self.location, Unset):
            location = []
            for location_item_data in self.location:
                location_item = location_item_data

                location.append(location_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if person is not UNSET:
            field_dict["person"] = person
        if location is not UNSET:
            field_dict["location"] = location

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        person = []
        _person = d.pop("person", UNSET)
        for person_item_data in _person or []:
            person_item = cast(List[int], person_item_data)

            person.append(person_item)

        location = []
        _location = d.pop("location", UNSET)
        for location_item_data in _location or []:
            location_item = cast(List[int], location_item_data)

            location.append(location_item)

        content_item_mentions_item = cls(
            person=person,
            location=location,
        )

        return content_item_mentions_item



from enum import Enum
from typing import Literal


class FindContentItemResolve(str, Enum):
    COLLECTION = "collection"
    TAGS = "tags"

    def __str__(self) -> str:
        return str(self.value)


FindContentItemResolveLiteral = Literal[
    "collection",
    "tags",
]



from enum import Enum
from typing import Literal


class SearchGroupBy(str, Enum):
    ARTICLES = "articles"
    RAW = "raw"

    def __str__(self) -> str:
        return str(self.value)


SearchGroupByLiteral = Literal[
    "articles",
    "raw",
]



from typing import Any, Union

from pandas import DataFrame, json_normalize

from impresso.api_client.api.content_items import find_content_item, get_content_item
from impresso.api_client.models.find_content_item_order_by import (
    FindContentItemOrderBy,
    FindContentItemOrderByLiteral,
)
from impresso.api_client.models.find_content_item_resolve import (
    FindContentItemResolve,
    FindContentItemResolveLiteral,
)
from impresso.api_client.types import UNSET, Unset
from impresso.api_models import ContentItem, BaseFind
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error
from impresso.util.py import get_enum_from_literal


class ContentItemsResponseSchema(BaseFind):
    """Schema for the content items response."""

    data: list[ContentItem]


class ContentItemsDataContainer(DataContainer):
    """Response of a content item call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        return json_normalize(self._data.to_dict()["data"]).set_index("uid")


class ContentItemDataContainer(DataContainer):
    """Response of a get content item call."""

    @property
    def raw(self) -> dict[str, Any]:
        """Return the data as a python dictionary."""
        return self._data.to_dict()

    @property
    def pydantic(self) -> ContentItem:
        """Return the data as a pydantic model."""
        return self._pydantic_model.model_validate(self.raw)

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        return json_normalize([self.raw])

    @property
    def size(self) -> int:
        """Current page size."""
        data = self._data.to_dict()
        if len(data):
            return 1
        return 0

    @property
    def total(self) -> int:
        """Total number of results."""
        return self.size


class ContentItemsResource(Resource):
    """Get content items from the impresso database."""

    name = "content_items"

    def find(
        self,
        resolve: Union[Unset, FindContentItemResolveLiteral] = UNSET,
        order_by: Union[Unset, FindContentItemOrderByLiteral] = UNSET,
        limit: Union[Unset, int] = UNSET,
        offset: Union[Unset, int] = UNSET,
    ):
        result = find_content_item.sync(
            client=self._api_client,
            resolve=get_enum_from_literal(resolve, FindContentItemResolve),
            order_by=get_enum_from_literal(order_by, FindContentItemOrderBy),
            limit=limit,
            offset=offset,
        )
        raise_for_error(result)
        return ContentItemsDataContainer(result, ContentItemsResponseSchema)

    def get(self, id: str):
        result = get_content_item.sync(client=self._api_client, id=id)
        raise_for_error(result)

        id_parts = id.split("-")
        issue_id = "-".join(id_parts[:-1])
        article_id = id_parts[-1]

        return ContentItemDataContainer(
            result,
            ContentItem,
            f"{self._get_web_app_base_url()}/issue/{issue_id}/view?articleId={article_id}",
        )



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="TextReusePassageClusterDetails")


@_attrs_define
class TextReusePassageClusterDetails:
    """Details of the cluster the passage belongs to

    Attributes:
        id (str): ID of the cluster
        cluster_size (Union[Unset, int]): The size of the cluster
        time_difference_day (Union[Unset, int]): The time difference in days between the two articles
        lexical_overlap (Union[Unset, float]): The lexical overlap between the two articles
    """

    id: str
    cluster_size: Union[Unset, int] = UNSET
    time_difference_day: Union[Unset, int] = UNSET
    lexical_overlap: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        cluster_size = self.cluster_size

        time_difference_day = self.time_difference_day

        lexical_overlap = self.lexical_overlap

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
            }
        )
        if cluster_size is not UNSET:
            field_dict["clusterSize"] = cluster_size
        if time_difference_day is not UNSET:
            field_dict["timeDifferenceDay"] = time_difference_day
        if lexical_overlap is not UNSET:
            field_dict["lexicalOverlap"] = lexical_overlap

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        cluster_size = d.pop("clusterSize", UNSET)

        time_difference_day = d.pop("timeDifferenceDay", UNSET)

        lexical_overlap = d.pop("lexicalOverlap", UNSET)

        text_reuse_passage_cluster_details = cls(
            id=id,
            cluster_size=cluster_size,
            time_difference_day=time_difference_day,
            lexical_overlap=lexical_overlap,
        )

        return text_reuse_passage_cluster_details



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="TopicRelatedTopicsItem")


@_attrs_define
class TopicRelatedTopicsItem:
    """
    Attributes:
        uid (str): The unique identifier of the related topic
        w (float): TODO
        avg (Union[Unset, float]): TODO
    """

    uid: str
    w: float
    avg: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        w = self.w

        avg = self.avg

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "w": w,
            }
        )
        if avg is not UNSET:
            field_dict["avg"] = avg

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid")

        w = d.pop("w")

        avg = d.pop("avg", UNSET)

        topic_related_topics_item = cls(
            uid=uid,
            w=w,
            avg=avg,
        )

        return topic_related_topics_item



from enum import Enum
from typing import Literal


class FindTextReusePassagesGroupBy(str, Enum):
    TEXTREUSECLUSTERID = "textReuseClusterId"

    def __str__(self) -> str:
        return str(self.value)


FindTextReusePassagesGroupByLiteral = Literal["textReuseClusterId",]



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.topic import Topic


T = TypeVar("T", bound="ContentItemTopic")


@_attrs_define
class ContentItemTopic:
    """TODO

    Attributes:
        relevance (float): TODO
        topic (Union[Unset, Topic]): A topic (TODO)
        topic_uid (Union[Unset, str]): TODO
    """

    relevance: float
    topic: Union[Unset, "Topic"] = UNSET
    topic_uid: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        relevance = self.relevance

        topic: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.topic, Unset):
            topic = self.topic.to_dict()

        topic_uid = self.topic_uid

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "relevance": relevance,
            }
        )
        if topic is not UNSET:
            field_dict["topic"] = topic
        if topic_uid is not UNSET:
            field_dict["topicUid"] = topic_uid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.topic import Topic

        d = src_dict.copy()
        relevance = d.pop("relevance")

        _topic = d.pop("topic", UNSET)
        topic: Union[Unset, Topic]
        if isinstance(_topic, Unset):
            topic = UNSET
        else:
            topic = Topic.from_dict(_topic)

        topic_uid = d.pop("topicUid", UNSET)

        content_item_topic = cls(
            relevance=relevance,
            topic=topic,
            topic_uid=topic_uid,
        )

        return content_item_topic



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_user import BaseUser


T = TypeVar("T", bound="Collection")


@_attrs_define
class Collection:
    """Description of the collection object (Collection class)

    Attributes:
        uid (str):
        name (str):
        description (str):
        status (str):  Example: PRI.
        creation_date (datetime.datetime):
        last_modified_date (datetime.datetime):
        count_items (Union[int, str]):
        creator (BaseUser):
        labels (Union[Unset, List[str]]):
    """

    uid: str
    name: str
    description: str
    status: str
    creation_date: datetime.datetime
    last_modified_date: datetime.datetime
    count_items: Union[int, str]
    creator: "BaseUser"
    labels: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        name = self.name

        description = self.description

        status = self.status

        creation_date = self.creation_date.isoformat()

        last_modified_date = self.last_modified_date.isoformat()

        count_items: Union[int, str]
        count_items = self.count_items

        creator = self.creator.to_dict()

        labels: Union[Unset, List[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "name": name,
                "description": description,
                "status": status,
                "creationDate": creation_date,
                "lastModifiedDate": last_modified_date,
                "countItems": count_items,
                "creator": creator,
            }
        )
        if labels is not UNSET:
            field_dict["labels"] = labels

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_user import BaseUser

        d = src_dict.copy()
        uid = d.pop("uid")

        name = d.pop("name")

        description = d.pop("description")

        status = d.pop("status")

        creation_date = isoparse(d.pop("creationDate"))

        last_modified_date = isoparse(d.pop("lastModifiedDate"))

        def _parse_count_items(data: object) -> Union[int, str]:
            return cast(Union[int, str], data)

        count_items = _parse_count_items(d.pop("countItems"))

        creator = BaseUser.from_dict(d.pop("creator"))

        labels = cast(List[str], d.pop("labels", UNSET))

        collection = cls(
            uid=uid,
            name=name,
            description=description,
            status=status,
            creation_date=creation_date,
            last_modified_date=last_modified_date,
            count_items=count_items,
            creator=creator,
            labels=labels,
        )

        return collection



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="TextReusePassageIssue")


@_attrs_define
class TextReusePassageIssue:
    """Issue details

    Attributes:
        id (str): ID of the issue
    """

    id: str

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        text_reuse_passage_issue = cls(
            id=id,
        )

        return text_reuse_passage_issue



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.page_regions_item import PageRegionsItem


T = TypeVar("T", bound="Page")


@_attrs_define
class Page:
    """A page of an article

    Attributes:
        uid (str): The unique identifier of the page
        num (int): The number of the page
        issue_uid (str): Reference to the article
        newspaper_uid (str): Unique ID of the newspaper
        iiif (str): The IIF image file name of the page
        iiif_thumbnail (str): The IIF image thumbnail file name of the page
        access_rights (str): The access rights code
        labels (List[str]): Page labels
        has_coords (bool): Whether the page has coordinates
        has_errors (bool): Whether the page has errors
        regions (List['PageRegionsItem']): Regions of the page
        obfuscated (Union[Unset, bool]): Whether the page image has been obfuscated because the user is not authorised
            to access it
        iiif_fragment (Union[Unset, str]): The IIIF fragment of the page, image file name
    """

    uid: str
    num: int
    issue_uid: str
    newspaper_uid: str
    iiif: str
    iiif_thumbnail: str
    access_rights: str
    labels: List[str]
    has_coords: bool
    has_errors: bool
    regions: List["PageRegionsItem"]
    obfuscated: Union[Unset, bool] = UNSET
    iiif_fragment: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        num = self.num

        issue_uid = self.issue_uid

        newspaper_uid = self.newspaper_uid

        iiif = self.iiif

        iiif_thumbnail = self.iiif_thumbnail

        access_rights = self.access_rights

        labels = self.labels

        has_coords = self.has_coords

        has_errors = self.has_errors

        regions = []
        for regions_item_data in self.regions:
            regions_item = regions_item_data.to_dict()
            regions.append(regions_item)

        obfuscated = self.obfuscated

        iiif_fragment = self.iiif_fragment

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "num": num,
                "issueUid": issue_uid,
                "newspaperUid": newspaper_uid,
                "iiif": iiif,
                "iiifThumbnail": iiif_thumbnail,
                "accessRights": access_rights,
                "labels": labels,
                "hasCoords": has_coords,
                "hasErrors": has_errors,
                "regions": regions,
            }
        )
        if obfuscated is not UNSET:
            field_dict["obfuscated"] = obfuscated
        if iiif_fragment is not UNSET:
            field_dict["iiifFragment"] = iiif_fragment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.page_regions_item import PageRegionsItem

        d = src_dict.copy()
        uid = d.pop("uid")

        num = d.pop("num")

        issue_uid = d.pop("issueUid")

        newspaper_uid = d.pop("newspaperUid")

        iiif = d.pop("iiif")

        iiif_thumbnail = d.pop("iiifThumbnail")

        access_rights = d.pop("accessRights")

        labels = cast(List[str], d.pop("labels"))

        has_coords = d.pop("hasCoords")

        has_errors = d.pop("hasErrors")

        regions = []
        _regions = d.pop("regions")
        for regions_item_data in _regions:
            regions_item = PageRegionsItem.from_dict(regions_item_data)

            regions.append(regions_item)

        obfuscated = d.pop("obfuscated", UNSET)

        iiif_fragment = d.pop("iiifFragment", UNSET)

        page = cls(
            uid=uid,
            num=num,
            issue_uid=issue_uid,
            newspaper_uid=newspaper_uid,
            iiif=iiif,
            iiif_thumbnail=iiif_thumbnail,
            access_rights=access_rights,
            labels=labels,
            has_coords=has_coords,
            has_errors=has_errors,
            regions=regions,
            obfuscated=obfuscated,
            iiif_fragment=iiif_fragment,
        )

        return page



from pandas import DataFrame, json_normalize

from impresso.api_client.api.newspapers import find_newspapers
from impresso.api_client.models.find_newspapers_order_by import (
    FindNewspapersOrderBy,
    FindNewspapersOrderByLiteral,
)
from impresso.api_client.types import UNSET
from impresso.api_models import BaseFind, Newspaper
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error
from impresso.util.py import get_enum_from_literal


class FindNewspapersSchema(BaseFind):
    """Schema for the find newspapers response."""

    data: list[Newspaper]


class FindNewspapersContainer(DataContainer):
    """Response of a search call."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["data"]
        if len(data):
            return json_normalize(self._data.to_dict()["data"]).set_index("uid")
        return DataFrame()


class NewspapersResource(Resource):
    """Search newspapers"""

    name = "newspapers"

    def find(
        self,
        q: str | None = None,
        order_by: FindNewspapersOrderByLiteral | None = None,
        limit: int | None = None,
        offset: int | None = None,
    ) -> FindNewspapersContainer:

        result = find_newspapers.sync(
            client=self._api_client,
            q=q if q is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindNewspapersOrderBy)
                if order_by is not None
                else UNSET
            ),
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
        )
        raise_for_error(result)
        return FindNewspapersContainer(
            result,
            FindNewspapersSchema,
            web_app_search_result_url=_build_web_app_newspapers_url(
                base_url=self._get_web_app_base_url(),
                q=q,
                order_by=order_by,
            ),
        )


def _build_web_app_newspapers_url(
    base_url: str,
    q: str | None = None,
    order_by: FindNewspapersOrderByLiteral | None = None,
) -> str:
    query_params = {
        "orderBy": order_by,
        "q": q,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/newspapers"
    return f"{url}?{query_string}" if query_string else url



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.collectable_items_updated_response import CollectableItemsUpdatedResponse
from ...models.error import Error
from ...models.update_collectable_items import UpdateCollectableItems
from ...types import Response


def _get_kwargs(
    collection_id: str,
    id: int,
    *,
    body: UpdateCollectableItems,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "patch",
        "url": f"/collections/{collection_id}/items/{id}",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[CollectableItemsUpdatedResponse, Error]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = CollectableItemsUpdatedResponse.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = Error.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Error.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = Error.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = Error.from_dict(response.json())

        return response_422
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        response_429 = Error.from_dict(response.json())

        return response_429
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Error.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[CollectableItemsUpdatedResponse, Error]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    id: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: UpdateCollectableItems,
) -> Response[Union[CollectableItemsUpdatedResponse, Error]]:
    """Updates the resource identified by id using data.

    Args:
        collection_id (str):
        id (int):
        body (UpdateCollectableItems): Request to update collectible items in a collection

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CollectableItemsUpdatedResponse, Error]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        id=id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    id: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: UpdateCollectableItems,
) -> Optional[Union[CollectableItemsUpdatedResponse, Error]]:
    """Updates the resource identified by id using data.

    Args:
        collection_id (str):
        id (int):
        body (UpdateCollectableItems): Request to update collectible items in a collection

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CollectableItemsUpdatedResponse, Error]
    """

    return sync_detailed(
        collection_id=collection_id,
        id=id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    collection_id: str,
    id: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: UpdateCollectableItems,
) -> Response[Union[CollectableItemsUpdatedResponse, Error]]:
    """Updates the resource identified by id using data.

    Args:
        collection_id (str):
        id (int):
        body (UpdateCollectableItems): Request to update collectible items in a collection

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CollectableItemsUpdatedResponse, Error]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        id=id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    id: int,
    *,
    client: Union[AuthenticatedClient, Client],
    body: UpdateCollectableItems,
) -> Optional[Union[CollectableItemsUpdatedResponse, Error]]:
    """Updates the resource identified by id using data.

    Args:
        collection_id (str):
        id (int):
        body (UpdateCollectableItems): Request to update collectible items in a collection

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CollectableItemsUpdatedResponse, Error]
    """

    return (
        await asyncio_detailed(
            collection_id=collection_id,
            id=id,
            client=client,
            body=body,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.text_reuse_cluster_details_resolution import TextReuseClusterDetailsResolution
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.text_reuse_cluster_details_facets_item import TextReuseClusterDetailsFacetsItem


T = TypeVar("T", bound="TextReuseClusterDetails")


@_attrs_define
class TextReuseClusterDetails:
    """Extra details of the cluster

    Attributes:
        facets (List['TextReuseClusterDetailsFacetsItem']):
        resolution (Union[Unset, TextReuseClusterDetailsResolution]): Resolution for the 'date' facet
    """

    facets: List["TextReuseClusterDetailsFacetsItem"]
    resolution: Union[Unset, TextReuseClusterDetailsResolution] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        facets = []
        for facets_item_data in self.facets:
            facets_item = facets_item_data.to_dict()
            facets.append(facets_item)

        resolution: Union[Unset, str] = UNSET
        if not isinstance(self.resolution, Unset):
            resolution = self.resolution.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "facets": facets,
            }
        )
        if resolution is not UNSET:
            field_dict["resolution"] = resolution

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.text_reuse_cluster_details_facets_item import TextReuseClusterDetailsFacetsItem

        d = src_dict.copy()
        facets = []
        _facets = d.pop("facets")
        for facets_item_data in _facets:
            facets_item = TextReuseClusterDetailsFacetsItem.from_dict(facets_item_data)

            facets.append(facets_item)

        _resolution = d.pop("resolution", UNSET)
        resolution: Union[Unset, TextReuseClusterDetailsResolution]
        if isinstance(_resolution, Unset):
            resolution = UNSET
        else:
            resolution = TextReuseClusterDetailsResolution(_resolution)

        text_reuse_cluster_details = cls(
            facets=facets,
            resolution=resolution,
        )

        return text_reuse_cluster_details



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="ContentItemMatch")


@_attrs_define
class ContentItemMatch:
    """TODO

    Attributes:
        fragment (str): TODO
        coords (Union[Unset, List[float]]): TODO
        page_uid (Union[Unset, str]): TODO
        iiif (Union[Unset, str]): TODO
    """

    fragment: str
    coords: Union[Unset, List[float]] = UNSET
    page_uid: Union[Unset, str] = UNSET
    iiif: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        fragment = self.fragment

        coords: Union[Unset, List[float]] = UNSET
        if not isinstance(self.coords, Unset):
            coords = self.coords

        page_uid = self.page_uid

        iiif = self.iiif

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "fragment": fragment,
            }
        )
        if coords is not UNSET:
            field_dict["coords"] = coords
        if page_uid is not UNSET:
            field_dict["pageUid"] = page_uid
        if iiif is not UNSET:
            field_dict["iiif"] = iiif

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        fragment = d.pop("fragment")

        coords = cast(List[float], d.pop("coords", UNSET))

        page_uid = d.pop("pageUid", UNSET)

        iiif = d.pop("iiif", UNSET)

        content_item_match = cls(
            fragment=fragment,
            coords=coords,
            page_uid=page_uid,
            iiif=iiif,
        )

        return content_item_match



from enum import Enum
from typing import Literal


class FindContentItemOrderBy(str, Enum):
    DATE = "date"
    RELEVANCE = "relevance"
    VALUE_0 = "-date"
    VALUE_2 = "-relevance"

    def __str__(self) -> str:
        return str(self.value)


FindContentItemOrderByLiteral = Literal[
    "date",
    "relevance",
    "-date",
    "-relevance",
]



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.text_reuse_passage_article_details import TextReusePassageArticleDetails
    from ..models.text_reuse_passage_cluster_details import TextReusePassageClusterDetails
    from ..models.text_reuse_passage_connected_clusters_item import TextReusePassageConnectedClustersItem
    from ..models.text_reuse_passage_issue import TextReusePassageIssue


T = TypeVar("T", bound="TextReusePassage")


@_attrs_define
class TextReusePassage:
    """Represents a passage of text that was identified as a part of a text reuse cluster

    Attributes:
        id (str): ID of the text reuse passage Example: abc123.
        article (TextReusePassageArticleDetails): Details of the article the passage belongs to
        text_reuse_cluster (TextReusePassageClusterDetails): Details of the cluster the passage belongs to
        offset_start (Union[None, int]):
        offset_end (Union[None, int]):
        content (str): Textual content of the passage
        title (str): Title of the content item (article) where this passage was found
        page_numbers (List[int]): Numbers of the pages where the passage was found
        collections (List[str]): Collection IDs the passage belongs to
        connected_clusters (Union[Unset, List['TextReusePassageConnectedClustersItem']]):
        is_front (Union[Unset, bool]): TBD
        size (Union[Unset, int]): Size of the passage
        newspaper (Union[Unset, Any]):
        issue (Union[Unset, TextReusePassageIssue]): Issue details
        date (Union[Unset, datetime.datetime]): Date of the item (article) where this passage was found
        page_regions (Union[Unset, List[str]]): Bounding box of the passage in the page
    """

    id: str
    article: "TextReusePassageArticleDetails"
    text_reuse_cluster: "TextReusePassageClusterDetails"
    offset_start: Union[None, int]
    offset_end: Union[None, int]
    content: str
    title: str
    page_numbers: List[int]
    collections: List[str]
    connected_clusters: Union[Unset, List["TextReusePassageConnectedClustersItem"]] = UNSET
    is_front: Union[Unset, bool] = UNSET
    size: Union[Unset, int] = UNSET
    newspaper: Union[Unset, Any] = UNSET
    issue: Union[Unset, "TextReusePassageIssue"] = UNSET
    date: Union[Unset, datetime.datetime] = UNSET
    page_regions: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        article = self.article.to_dict()

        text_reuse_cluster = self.text_reuse_cluster.to_dict()

        offset_start: Union[None, int]
        offset_start = self.offset_start

        offset_end: Union[None, int]
        offset_end = self.offset_end

        content = self.content

        title = self.title

        page_numbers = self.page_numbers

        collections = self.collections

        connected_clusters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.connected_clusters, Unset):
            connected_clusters = []
            for connected_clusters_item_data in self.connected_clusters:
                connected_clusters_item = connected_clusters_item_data.to_dict()
                connected_clusters.append(connected_clusters_item)

        is_front = self.is_front

        size = self.size

        newspaper = self.newspaper

        issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.issue, Unset):
            issue = self.issue.to_dict()

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        page_regions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.page_regions, Unset):
            page_regions = self.page_regions

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "article": article,
                "textReuseCluster": text_reuse_cluster,
                "offsetStart": offset_start,
                "offsetEnd": offset_end,
                "content": content,
                "title": title,
                "pageNumbers": page_numbers,
                "collections": collections,
            }
        )
        if connected_clusters is not UNSET:
            field_dict["connectedClusters"] = connected_clusters
        if is_front is not UNSET:
            field_dict["isFront"] = is_front
        if size is not UNSET:
            field_dict["size"] = size
        if newspaper is not UNSET:
            field_dict["newspaper"] = newspaper
        if issue is not UNSET:
            field_dict["issue"] = issue
        if date is not UNSET:
            field_dict["date"] = date
        if page_regions is not UNSET:
            field_dict["pageRegions"] = page_regions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.text_reuse_passage_article_details import TextReusePassageArticleDetails
        from ..models.text_reuse_passage_cluster_details import TextReusePassageClusterDetails
        from ..models.text_reuse_passage_connected_clusters_item import TextReusePassageConnectedClustersItem
        from ..models.text_reuse_passage_issue import TextReusePassageIssue

        d = src_dict.copy()
        id = d.pop("id")

        article = TextReusePassageArticleDetails.from_dict(d.pop("article"))

        text_reuse_cluster = TextReusePassageClusterDetails.from_dict(d.pop("textReuseCluster"))

        def _parse_offset_start(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        offset_start = _parse_offset_start(d.pop("offsetStart"))

        def _parse_offset_end(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        offset_end = _parse_offset_end(d.pop("offsetEnd"))

        content = d.pop("content")

        title = d.pop("title")

        page_numbers = cast(List[int], d.pop("pageNumbers"))

        collections = cast(List[str], d.pop("collections"))

        connected_clusters = []
        _connected_clusters = d.pop("connectedClusters", UNSET)
        for connected_clusters_item_data in _connected_clusters or []:
            connected_clusters_item = TextReusePassageConnectedClustersItem.from_dict(connected_clusters_item_data)

            connected_clusters.append(connected_clusters_item)

        is_front = d.pop("isFront", UNSET)

        size = d.pop("size", UNSET)

        newspaper = d.pop("newspaper", UNSET)

        _issue = d.pop("issue", UNSET)
        issue: Union[Unset, TextReusePassageIssue]
        if isinstance(_issue, Unset):
            issue = UNSET
        else:
            issue = TextReusePassageIssue.from_dict(_issue)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.datetime]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date)

        page_regions = cast(List[str], d.pop("pageRegions", UNSET))

        text_reuse_passage = cls(
            id=id,
            article=article,
            text_reuse_cluster=text_reuse_cluster,
            offset_start=offset_start,
            offset_end=offset_end,
            content=content,
            title=title,
            page_numbers=page_numbers,
            collections=collections,
            connected_clusters=connected_clusters,
            is_front=is_front,
            size=size,
            newspaper=newspaper,
            issue=issue,
            date=date,
            page_regions=page_regions,
        )

        return text_reuse_passage



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.newspaper_issue import NewspaperIssue
    from ..models.newspaper_property import NewspaperProperty


T = TypeVar("T", bound="Newspaper")


@_attrs_define
class Newspaper:
    """A newspaper

    Attributes:
        uid (str): The unique identifier of the newspaper
        acronym (str): The acronym of the newspaper
        labels (List[str]): The labels of the newspaper
        languages (List[str]): Language codes of the languages used in the newspaper
        included (bool): TODO
        name (str): Title of the newspaper
        end_year (Union[None, int]):
        start_year (Union[None, int]):
        count_articles (int): The number of articles in the newspaper
        count_issues (int): The number of issues in the newspaper
        count_pages (int): The number of pages in the newspaper
        delta_year (int): The number of years of the newspaper available
        properties (Union[Unset, List['NewspaperProperty']]): TODO
        first_issue (Union[Unset, NewspaperIssue]):
        last_issue (Union[Unset, NewspaperIssue]):
        fetched (Union[Unset, bool]): TODO
    """

    uid: str
    acronym: str
    labels: List[str]
    languages: List[str]
    included: bool
    name: str
    end_year: Union[None, int]
    start_year: Union[None, int]
    count_articles: int
    count_issues: int
    count_pages: int
    delta_year: int
    properties: Union[Unset, List["NewspaperProperty"]] = UNSET
    first_issue: Union[Unset, "NewspaperIssue"] = UNSET
    last_issue: Union[Unset, "NewspaperIssue"] = UNSET
    fetched: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        acronym = self.acronym

        labels = self.labels

        languages = self.languages

        included = self.included

        name = self.name

        end_year: Union[None, int]
        end_year = self.end_year

        start_year: Union[None, int]
        start_year = self.start_year

        count_articles = self.count_articles

        count_issues = self.count_issues

        count_pages = self.count_pages

        delta_year = self.delta_year

        properties: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = []
            for properties_item_data in self.properties:
                properties_item = properties_item_data.to_dict()
                properties.append(properties_item)

        first_issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.first_issue, Unset):
            first_issue = self.first_issue.to_dict()

        last_issue: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.last_issue, Unset):
            last_issue = self.last_issue.to_dict()

        fetched = self.fetched

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "uid": uid,
                "acronym": acronym,
                "labels": labels,
                "languages": languages,
                "included": included,
                "name": name,
                "endYear": end_year,
                "startYear": start_year,
                "countArticles": count_articles,
                "countIssues": count_issues,
                "countPages": count_pages,
                "deltaYear": delta_year,
            }
        )
        if properties is not UNSET:
            field_dict["properties"] = properties
        if first_issue is not UNSET:
            field_dict["firstIssue"] = first_issue
        if last_issue is not UNSET:
            field_dict["lastIssue"] = last_issue
        if fetched is not UNSET:
            field_dict["fetched"] = fetched

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.newspaper_issue import NewspaperIssue
        from ..models.newspaper_property import NewspaperProperty

        d = src_dict.copy()
        uid = d.pop("uid")

        acronym = d.pop("acronym")

        labels = cast(List[str], d.pop("labels"))

        languages = cast(List[str], d.pop("languages"))

        included = d.pop("included")

        name = d.pop("name")

        def _parse_end_year(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        end_year = _parse_end_year(d.pop("endYear"))

        def _parse_start_year(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        start_year = _parse_start_year(d.pop("startYear"))

        count_articles = d.pop("countArticles")

        count_issues = d.pop("countIssues")

        count_pages = d.pop("countPages")

        delta_year = d.pop("deltaYear")

        properties = []
        _properties = d.pop("properties", UNSET)
        for properties_item_data in _properties or []:
            properties_item = NewspaperProperty.from_dict(properties_item_data)

            properties.append(properties_item)

        _first_issue = d.pop("firstIssue", UNSET)
        first_issue: Union[Unset, NewspaperIssue]
        if isinstance(_first_issue, Unset):
            first_issue = UNSET
        else:
            first_issue = NewspaperIssue.from_dict(_first_issue)

        _last_issue = d.pop("lastIssue", UNSET)
        last_issue: Union[Unset, NewspaperIssue]
        if isinstance(_last_issue, Unset):
            last_issue = UNSET
        else:
            last_issue = NewspaperIssue.from_dict(_last_issue)

        fetched = d.pop("fetched", UNSET)

        newspaper = cls(
            uid=uid,
            acronym=acronym,
            labels=labels,
            languages=languages,
            included=included,
            name=name,
            end_year=end_year,
            start_year=start_year,
            count_articles=count_articles,
            count_issues=count_issues,
            count_pages=count_pages,
            delta_year=delta_year,
            properties=properties,
            first_issue=first_issue,
            last_issue=last_issue,
            fetched=fetched,
        )

        return newspaper



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.text_reuse_cluster import TextReuseCluster
    from ..models.text_reuse_cluster_details import TextReuseClusterDetails


T = TypeVar("T", bound="TextReuseClusterCompound")


@_attrs_define
class TextReuseClusterCompound:
    """Text reuse cluster with details and a sample

    Attributes:
        text_sample (str):
        cluster (Union[Unset, TextReuseCluster]): Represents a cluster of text reuse passages
        details (Union[Unset, TextReuseClusterDetails]): Extra details of the cluster
    """

    text_sample: str
    cluster: Union[Unset, "TextReuseCluster"] = UNSET
    details: Union[Unset, "TextReuseClusterDetails"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        text_sample = self.text_sample

        cluster: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cluster, Unset):
            cluster = self.cluster.to_dict()

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "textSample": text_sample,
            }
        )
        if cluster is not UNSET:
            field_dict["cluster"] = cluster
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.text_reuse_cluster import TextReuseCluster
        from ..models.text_reuse_cluster_details import TextReuseClusterDetails

        d = src_dict.copy()
        text_sample = d.pop("textSample")

        _cluster = d.pop("cluster", UNSET)
        cluster: Union[Unset, TextReuseCluster]
        if isinstance(_cluster, Unset):
            cluster = UNSET
        else:
            cluster = TextReuseCluster.from_dict(_cluster)

        _details = d.pop("details", UNSET)
        details: Union[Unset, TextReuseClusterDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = TextReuseClusterDetails.from_dict(_details)

        text_reuse_cluster_compound = cls(
            text_sample=text_sample,
            cluster=cluster,
            details=details,
        )

        return text_reuse_cluster_compound



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.location_specific_fields_descriptions import LocationSpecificFieldsDescriptions
    from ..models.location_specific_fields_images_item import LocationSpecificFieldsImagesItem
    from ..models.location_specific_fields_labels import LocationSpecificFieldsLabels


T = TypeVar("T", bound="LocationSpecificFields")


@_attrs_define
class LocationSpecificFields:
    """Details of a wikidata entity

    Attributes:
        id (str):
        type (str):
        labels (LocationSpecificFieldsLabels): Labels of the entity. Key is the language code.
        descriptions (LocationSpecificFieldsDescriptions): Labels of the entity. Key is the language code.
        images (List['LocationSpecificFieldsImagesItem']):
    """

    id: str
    type: str
    labels: "LocationSpecificFieldsLabels"
    descriptions: "LocationSpecificFieldsDescriptions"
    images: List["LocationSpecificFieldsImagesItem"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        type = self.type

        labels = self.labels.to_dict()

        descriptions = self.descriptions.to_dict()

        images = []
        for images_item_data in self.images:
            images_item = images_item_data.to_dict()
            images.append(images_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "type": type,
                "labels": labels,
                "descriptions": descriptions,
                "images": images,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.location_specific_fields_descriptions import LocationSpecificFieldsDescriptions
        from ..models.location_specific_fields_images_item import LocationSpecificFieldsImagesItem
        from ..models.location_specific_fields_labels import LocationSpecificFieldsLabels

        d = src_dict.copy()
        id = d.pop("id")

        type = d.pop("type")

        labels = LocationSpecificFieldsLabels.from_dict(d.pop("labels"))

        descriptions = LocationSpecificFieldsDescriptions.from_dict(d.pop("descriptions"))

        images = []
        _images = d.pop("images")
        for images_item_data in _images:
            images_item = LocationSpecificFieldsImagesItem.from_dict(images_item_data)

            images.append(images_item)

        location_specific_fields = cls(
            id=id,
            type=type,
            labels=labels,
            descriptions=descriptions,
            images=images,
        )

        location_specific_fields.additional_properties = d
        return location_specific_fields

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum
from typing import Literal


class GetSearchFacetRangeInclude(str, Enum):
    ALL = "all"
    EDGE = "edge"
    UPPER = "upper"

    def __str__(self) -> str:
        return str(self.value)


GetSearchFacetRangeIncludeLiteral = Literal[
    "all",
    "edge",
    "upper",
]



from pandas import DataFrame, json_normalize
from impresso.api_client.api.tools import perform_ner
from impresso.api_client.models.impresso_named_entity_recognition_request import (
    ImpressoNamedEntityRecognitionRequest,
)
from impresso.api_client.models.impresso_named_entity_recognition_request_method import (
    ImpressoNamedEntityRecognitionRequestMethod,
)
from impresso.api_models import ImpressoNerResponse
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.util.error import raise_for_error


class ImpressoNerSchema(ImpressoNerResponse):
    """Schema for the find entities response."""

    pass


class NerContainer(DataContainer):
    """Name entity recognition result container."""

    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["entities"]
        if len(data):
            return json_normalize(self._data.to_dict()["entities"]).set_index("id")
        return DataFrame()

    @property
    def total(self) -> int:
        """Total number of results."""
        return len(self.raw.get("entities", []))

    @property
    def limit(self) -> int:
        """Page size."""
        return len(self.raw.get("entities", []))

    @property
    def offset(self) -> int:
        """Page offset."""
        return 0

    @property
    def size(self) -> int:
        """Current page size."""
        return len(self.raw.get("entities", []))


class ToolsResource(Resource):
    """Various helper tools"""

    name = "tools"

    def ner(self, text: str) -> NerContainer:
        """Named Entity Recognition

        This method is faster than `ner_nel` but does not provide any linking to external resources.

        Args:
            text (str): Text to process

        Returns:
            NerContainer: List of named entities
        """
        result = perform_ner.sync(
            client=self._api_client,
            body=ImpressoNamedEntityRecognitionRequest(
                text=text, method=ImpressoNamedEntityRecognitionRequestMethod.NER
            ),
        )
        raise_for_error(result)

        return NerContainer(
            result,
            ImpressoNerSchema,
            web_app_search_result_url=None,
        )

    def ner_nel(self, text: str) -> NerContainer:
        """Named Entity Recognition and Named Entity Linking

        This method is slower than `ner` but provides linking to external resources.

        Args:
            text (str): Text to process

        Returns:
            NerContainer: List of named entities
        """
        result = perform_ner.sync(
            client=self._api_client,
            body=ImpressoNamedEntityRecognitionRequest(
                text=text, method=ImpressoNamedEntityRecognitionRequestMethod.NER_NEL
            ),
        )
        raise_for_error(result)

        return NerContainer(
            result,
            ImpressoNerSchema,
            web_app_search_result_url=None,
        )

    def nel(self, text: str) -> NerContainer:
        """Named Entity Linking

        This method requires named entities to be enclosed in tags: [START]entity[END].

        Args:
            text (str): Text to process

        Returns:
            NerContainer: List of named entities
        """
        result = perform_ner.sync(
            client=self._api_client,
            body=ImpressoNamedEntityRecognitionRequest(
                text=text, method=ImpressoNamedEntityRecognitionRequestMethod.NEL
            ),
        )
        raise_for_error(result)

        return NerContainer(
            result,
            ImpressoNerSchema,
            web_app_search_result_url=None,
        )



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="YearWeights")


@_attrs_define
class YearWeights:
    """Total items counts within a year

    Attributes:
        c (Union[Unset, float]): Number of content items
        a (Union[Unset, float]): Number of articles
        p (Union[Unset, float]): Number of pages
        i (Union[Unset, float]): Number of issues
        m (Union[Unset, float]): Number of images (with or without vectors)
    """

    c: Union[Unset, float] = UNSET
    a: Union[Unset, float] = UNSET
    p: Union[Unset, float] = UNSET
    i: Union[Unset, float] = UNSET
    m: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        c = self.c

        a = self.a

        p = self.p

        i = self.i

        m = self.m

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if c is not UNSET:
            field_dict["c"] = c
        if a is not UNSET:
            field_dict["a"] = a
        if p is not UNSET:
            field_dict["p"] = p
        if i is not UNSET:
            field_dict["i"] = i
        if m is not UNSET:
            field_dict["m"] = m

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        c = d.pop("c", UNSET)

        a = d.pop("a", UNSET)

        p = d.pop("p", UNSET)

        i = d.pop("i", UNSET)

        m = d.pop("m", UNSET)

        year_weights = cls(
            c=c,
            a=a,
            p=p,
            i=i,
            m=m,
        )

        return year_weights



from pandas import DataFrame, json_normalize

from impresso.api_client.api.search_facets import get_tr_clusters_facet
from impresso.api_client.api.text_reuse_clusters import find_text_reuse_clusters
from impresso.api_client.models.find_text_reuse_clusters_order_by import (
    FindTextReuseClustersOrderBy,
    FindTextReuseClustersOrderByLiteral,
)
from impresso.api_client.models.get_tr_clusters_facet_id import (
    GetTrClustersFacetId,
    GetTrClustersFacetIdLiteral,
)
from impresso.api_client.models.get_tr_clusters_facet_order_by import (
    GetTrClustersFacetOrderBy,
    GetTrClustersFacetOrderByLiteral,
)
from impresso.api_client.types import UNSET, Unset
from impresso.api_models import (
    BaseFind,
    Filter,
    Q,
    SearchFacet,
    TextReuseClusterCompound,
)
from impresso.data_container import DataContainer
from impresso.resources.base import Resource
from impresso.resources.search import FacetDataContainer
from impresso.structures import AND, OR, DateRange
from impresso.util.error import raise_for_error
from impresso.util.filters import and_or_filter, filters_as_protobuf
from impresso.util.py import get_enum_from_literal


class FindTextReuseClusterResponseSchema(BaseFind):
    """Schema for the text reuse clusters response."""

    data: list[TextReuseClusterCompound]


class FindTextReuseClustersContainer(DataContainer):
    @property
    def df(self) -> DataFrame:
        """Return the data as a pandas dataframe."""
        data = self._data.to_dict()["clusters"]
        if len(data):
            return json_normalize(data).set_index("cluster.id")
        return DataFrame()

    @property
    def pydantic(self):
        """Return the data as a pydantic model."""
        remapped_raw = {
            "data": self.raw.get("clusters", []),
            "info": self.raw.get("info", {}),
            "total": self.total,
            "limit": self.limit,
            "offset": self.offset,
        }
        return self._pydantic_model.model_validate(remapped_raw)

    @property
    def size(self) -> int:
        """Current page size."""
        return len(self.raw.get("clusters", []))

    @property
    def total(self) -> int:
        """Total number of results."""
        return self.raw.get("info", {}).get("total", 0)

    @property
    def limit(self) -> int:
        """Page size."""
        return self.raw.get("info", {}).get("limit", 0)

    @property
    def offset(self) -> int:
        """Page offset."""
        return self.raw.get("info", {}).get("offset", 0)


Range = tuple[int, int]


class TextReuseClustersResource(Resource):
    """Text reuse clusters resource."""

    name = "textReuseClusters"

    def find(
        self,
        text: str | None = None,
        title: str | AND[str] | OR[str] | None = None,
        order_by: FindTextReuseClustersOrderByLiteral | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        collection_id: str | OR[str] | None = None,
        limit: int | None = None,
        offset: int | None = None,
        front_page: bool | None = None,
        topic_id: str | AND[str] | OR[str] | None = None,
        language: str | OR[str] | None = None,
        country: str | OR[str] | None = None,
        mention: str | AND[str] | OR[str] | None = None,
        entity_id: str | AND[str] | OR[str] | None = None,
    ) -> FindTextReuseClustersContainer:

        filters = _build_filters(
            text=text,
            cluster_size=cluster_size,
            title=title,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
            collection_id=collection_id,
            front_page=front_page,
            topic_id=topic_id,
            language=language,
            country=country,
            mention=mention,
            entity_id=entity_id,
        )
        filters_pb = filters_as_protobuf(filters or [])

        result = find_text_reuse_clusters.sync(
            client=self._api_client,
            limit=limit if limit is not None else UNSET,
            offset=offset if offset is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, FindTextReuseClustersOrderBy)
                if order_by is not None
                else UNSET
            ),
            filters=filters_pb if filters_pb else UNSET,
        )
        raise_for_error(result)
        return FindTextReuseClustersContainer(
            result,
            FindTextReuseClusterResponseSchema,
            web_app_search_result_url=_build_web_app_find_clusters_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )

    def facet(
        self,
        facet: GetTrClustersFacetIdLiteral,
        order_by: GetTrClustersFacetOrderByLiteral | None = "value",
        limit: int | None = None,
        offset: int | None = None,
        cluster_size: Range | AND[Range] | OR[Range] | None = None,
        date_range: DateRange | None = None,
        newspaper_id: str | OR[str] | None = None,
        lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
        day_delta: Range | AND[Range] | OR[Range] | None = None,
    ) -> FacetDataContainer:
        facet_id = get_enum_from_literal(facet, GetTrClustersFacetId)
        if isinstance(facet_id, Unset):
            raise ValueError(f"{facet} is not a valid value")

        filters = _build_cluster_facet_filters(
            cluster_size=cluster_size,
            lexical_overlap=lexical_overlap,
            day_delta=day_delta,
            date_range=date_range,
            newspaper_id=newspaper_id,
        )

        filters_pb = filters_as_protobuf(filters or [])

        result = get_tr_clusters_facet.sync(
            client=self._api_client,
            id=facet_id,
            filters=filters_pb if filters_pb else UNSET,
            offset=offset if offset is not None else UNSET,
            limit=limit if limit is not None else UNSET,
            order_by=(
                get_enum_from_literal(order_by, GetTrClustersFacetOrderBy)
                if order_by is not None
                else UNSET
            ),
        )
        raise_for_error(result)
        return FacetDataContainer(
            result,
            SearchFacet,
            limit=limit,
            offset=offset,
            web_app_search_result_url=_build_web_app_find_clusters_url(
                base_url=self._get_web_app_base_url(),
                filters=filters_pb,
                limit=limit,
                offset=offset,
                order_by=order_by,
            ),
        )


def _build_cluster_facet_filters(
    cluster_size: Range | AND[Range] | OR[Range] | None = None,
    date_range: DateRange | None = None,
    newspaper_id: str | OR[str] | None = None,
    lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
    day_delta: Range | AND[Range] | OR[Range] | None = None,
) -> list[Filter]:
    """Build text reuse clusters facet filters."""

    filters: list[Filter] = []
    if cluster_size is not None:
        filters.extend(
            and_or_filter(
                cluster_size,
                "text_reuse_cluster_size",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if date_range is not None:
        filters.append(
            Filter(
                type="daterange",
                q=Q(DateRange._as_filter_value(date_range)),
                context="exclude" if date_range.inverted else "include",
                daterange=None,
            )
        )
    if newspaper_id is not None:
        filters.extend(and_or_filter(newspaper_id, "newspaper"))
    if lexical_overlap is not None:
        filters.extend(
            and_or_filter(
                lexical_overlap,
                "text_reuse_cluster_lexical_overlap",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if day_delta is not None:
        filters.extend(
            and_or_filter(
                day_delta,
                "text_reuse_cluster_day_delta",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    return filters


def _build_filters(
    text: str | None = None,
    cluster_id: str | AND[str] | OR[str] | None = None,
    cluster_size: Range | AND[Range] | OR[Range] | None = None,
    title: str | AND[str] | OR[str] | None = None,
    lexical_overlap: Range | AND[Range] | OR[Range] | None = None,
    day_delta: Range | AND[Range] | OR[Range] | None = None,
    date_range: DateRange | None = None,
    newspaper_id: str | OR[str] | None = None,
    collection_id: str | OR[str] | None = None,
    front_page: bool | None = None,
    topic_id: str | AND[str] | OR[str] | None = None,
    language: str | OR[str] | None = None,
    country: str | OR[str] | None = None,
    mention: str | AND[str] | OR[str] | None = None,
    entity_id: str | AND[str] | OR[str] | None = None,
) -> list[Filter]:
    """Build text reuse clusters filters."""

    filters: list[Filter] = []
    if text is not None:
        filters.extend(and_or_filter(text, "string"))
    if cluster_id is not None:
        filters.extend(and_or_filter(cluster_id, "text_reuse_cluster"))
    if cluster_size is not None:
        filters.extend(
            and_or_filter(
                cluster_size,
                "text_reuse_cluster_size",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if title is not None:
        filters.extend(and_or_filter(title, "title"))
    if lexical_overlap is not None:
        filters.extend(
            and_or_filter(
                lexical_overlap,
                "text_reuse_cluster_lexical_overlap",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if day_delta is not None:
        filters.extend(
            and_or_filter(
                day_delta,
                "text_reuse_cluster_day_delta",
                lambda r: f"{r[0]} TO {r[1]}",
            )
        )
    if date_range is not None:
        filters.append(
            Filter(
                type="daterange",
                q=Q(DateRange._as_filter_value(date_range)),
                context="exclude" if date_range.inverted else "include",
                daterange=None,
            )
        )
    if newspaper_id is not None:
        filters.extend(and_or_filter(newspaper_id, "newspaper"))
    if collection_id is not None:
        filters.extend(and_or_filter(collection_id, "collection"))
    if front_page:
        filters.append(Filter(type="is_front", daterange=None))
    if topic_id is not None:
        filters.extend(and_or_filter(topic_id, "topic"))
    if language is not None:
        filters.extend(and_or_filter(language, "language"))
    if country is not None:
        filters.extend(and_or_filter(country, "country"))
    if mention is not None:
        filters.extend(and_or_filter(mention, "mention"))
    if entity_id is not None:
        filters.extend(and_or_filter(entity_id, "entity"))

    return filters


def _build_web_app_find_clusters_url(
    base_url: str,
    filters=str | None,
    limit=int | None,
    offset=int | None,
    order_by=GetTrClustersFacetOrderByLiteral | None,
) -> str:
    page = offset // limit if limit is not None and offset is not None else 0
    query_params = {
        "orderBy": order_by,
        "sq": filters,
        "p": page + 1,
    }
    query_string = "&".join(
        f"{key}={value}" for key, value in query_params.items() if value is not None
    )
    url = f"{base_url}/text-reuse/clusters"
    return f"{url}?{query_string}" if query_string else url



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="NewspaperProperty")


@_attrs_define
class NewspaperProperty:
    """
    Attributes:
        name (str): The name of the property
        value (str): The value of the property
        label (str): The label of the property
        is_url (Union[Unset, bool]): Whether the value is a URL
    """

    name: str
    value: str
    label: str
    is_url: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        value = self.value

        label = self.label

        is_url = self.is_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "value": value,
                "label": label,
            }
        )
        if is_url is not UNSET:
            field_dict["isUrl"] = is_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        value = d.pop("value")

        label = d.pop("label")

        is_url = d.pop("isUrl", UNSET)

        newspaper_property = cls(
            name=name,
            value=value,
            label=label,
            is_url=is_url,
        )

        newspaper_property.additional_properties = d
        return newspaper_property

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
